@@ -62,16 +62,16 @@ namespace Magick
   class MagickDLLDecl adaptiveThresholdImage : public std::unary_function<Image&,void>
   {
   public:
-    adaptiveThresholdImage( const unsigned int width_,
-                            const unsigned int height_,
-                            const unsigned int offset_ = 0  );
+    adaptiveThresholdImage( const size_t width_,
+                            const size_t height_,
+                            const ssize_t offset_ = 0  );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _width;
-    unsigned int _height;
-    unsigned int _offset;
+    size_t _width;
+    size_t _height;
+    ssize_t _offset;
   };
   
   // Add noise to image with specified noise type
@@ -234,6 +234,21 @@ namespace Magick
     Color _penColor;
   };
 
+  // Apply a color matrix to the image channels.  The user supplied
+  // matrix may be of order 1 to 5 (1x1 through 5x5).
+  class MagickDLLDecl colorMatrixImage : public std::unary_function<Image&,void>
+  {
+  public:
+    colorMatrixImage( const size_t order_,
+          const double *color_matrix_ );
+
+    void operator()( Image &image_ ) const;
+
+  private:
+    size_t  _order;
+    const double *_color_matrix;
+  };
+
   // Convert the image colorspace representation
   class MagickDLLDecl colorSpaceImage : public std::unary_function<Image&,void>
   {
@@ -264,8 +279,8 @@ namespace Magick
   {
   public:
     compositeImage( const Image &compositeImage_,
-        int xOffset_,
-        int yOffset_,
+        ssize_t xOffset_,
+        ssize_t yOffset_,
         CompositeOperator compose_ = InCompositeOp );
 
     compositeImage( const Image &compositeImage_,
@@ -276,8 +291,8 @@ namespace Magick
 
   private:
     Image             _compositeImage;
-    int               _xOffset;
-    int               _yOffset;
+    ssize_t               _xOffset;
+    ssize_t               _yOffset;
     CompositeOperator _compose;
   };
 
@@ -285,12 +300,12 @@ namespace Magick
   class MagickDLLDecl contrastImage : public std::unary_function<Image&,void>
   {
   public:
-    contrastImage( const unsigned int sharpen_ );
+    contrastImage( const size_t sharpen_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _sharpen;
+    size_t _sharpen;
   };
 
   // Crop image (subregion of original image)
@@ -309,12 +324,12 @@ namespace Magick
   class MagickDLLDecl cycleColormapImage : public std::unary_function<Image&,void>
   {
   public:
-    cycleColormapImage( const int amount_ );
+    cycleColormapImage( const ssize_t amount_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    int _amount;
+    ssize_t _amount;
   };
 
   // Despeckle image (reduce speckle noise)
@@ -336,19 +351,19 @@ namespace Magick
   {
   public:
     distortImage( const Magick::DistortImageMethod method_,
-      const unsigned long number_arguments_,
+      const size_t number_arguments_,
       const double *arguments_,
       const bool bestfit_ );
           
     distortImage( const Magick::DistortImageMethod method_,
-      const unsigned long number_arguments_,
+      const size_t number_arguments_,
       const double *arguments_ );
 
     void operator()( Image &image_ ) const;
 
   private:
     DistortImageMethod _method;
-    unsigned long _number_arguments;
+    size_t _number_arguments;
     const double *_arguments;
     bool _bestfit;
   };
@@ -448,8 +463,8 @@ namespace Magick
     // Flood-fill color across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillColorImage( const unsigned int x_,
-                         const unsigned int y_,
+    floodFillColorImage( const ssize_t x_,
+                         const ssize_t y_,
        const Color &fillColor_ );
 
     floodFillColorImage( const Geometry &point_,
@@ -458,8 +473,8 @@ namespace Magick
     // Flood-fill color across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillColorImage( const unsigned int x_,
-                         const unsigned int y_,
+    floodFillColorImage( const ssize_t x_,
+                         const ssize_t y_,
        const Color &fillColor_,
        const Color &borderColor_ );
 
@@ -470,8 +485,8 @@ namespace Magick
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int   _x;
-    unsigned int   _y;
+    ssize_t   _x;
+    ssize_t   _y;
     Color          _fillColor;
     Color          _borderColor;
   };
@@ -483,8 +498,8 @@ namespace Magick
     // Flood-fill texture across pixels that match the color of the
     // target pixel and are neighbors of the target pixel.
     // Uses current fuzz setting when determining color match.
-    floodFillTextureImage( const unsigned int x_,
-                           const unsigned int y_,
+    floodFillTextureImage( const ssize_t x_,
+                           const ssize_t y_,
          const Image &texture_ );
 
     floodFillTextureImage( const Geometry &point_,
@@ -493,8 +508,8 @@ namespace Magick
     // Flood-fill texture across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillTextureImage( const unsigned int x_,
-                           const unsigned int y_,
+    floodFillTextureImage( const ssize_t x_,
+                           const ssize_t y_,
          const Image &texture_,
          const Color &borderColor_ );
 
@@ -505,8 +520,8 @@ namespace Magick
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int  _x;
-    unsigned int  _y;
+    ssize_t  _x;
+    ssize_t  _y;
     Image         _texture;
     Color         _borderColor;
   };
@@ -528,16 +543,16 @@ namespace Magick
   public:
     frameImage( const Geometry &geometry_ = frameGeometryDefault );
 
-    frameImage( const unsigned int width_, const unsigned int height_,
-    const int innerBevel_ = 6, const int outerBevel_ = 6 );
+    frameImage( const size_t width_, const size_t height_,
+    const ssize_t innerBevel_ = 6, const ssize_t outerBevel_ = 6 );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _width;
-    unsigned int _height;
-    int          _outerBevel;
-    int          _innerBevel;
+    size_t _width;
+    size_t _height;
+    ssize_t          _outerBevel;
+    ssize_t          _innerBevel;
   };
 
   // Gamma correct image
@@ -702,7 +717,7 @@ namespace Magick
   public:
     matteFloodfillImage( const Color &target_ ,
        const unsigned int matte_,
-       const int x_, const int y_,
+       const ssize_t x_, const ssize_t y_,
        const PaintMethod method_ );
 
     void operator()( Image &image_ ) const;
@@ -710,8 +725,8 @@ namespace Magick
   private:
     Color         _target;
     unsigned int  _matte;
-    int           _x;
-    int           _y;
+    ssize_t           _x;
+    ssize_t           _y;
     PaintMethod   _method;
   };
 
@@ -851,33 +866,18 @@ namespace Magick
     bool       _raisedFlag;
   };
 
-  // Apply a color matrix to the image channels.  The user supplied
-  // matrix may be of order 1 to 5 (1x1 through 5x5).
-  class MagickDLLDecl recolorImage : public std::unary_function<Image&,void>
-  {
-  public:
-    recolorImage( const unsigned int order_,
-          const double *color_matrix_ );
-
-    void operator()( Image &image_ ) const;
-
-  private:
-    unsigned int  _order;
-    const double *_color_matrix;
-  };
-
   // Reduce noise in image using a noise peak elimination filter
   class MagickDLLDecl reduceNoiseImage : public std::unary_function<Image&,void>
   {
   public:
     reduceNoiseImage( void );
 
-    reduceNoiseImage (const  unsigned int order_ );
+    reduceNoiseImage (const  size_t order_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _order;
+    size_t _order;
   };
 
   // Resize image to specified size.
@@ -899,13 +899,13 @@ namespace Magick
   public:
     rollImage( const Geometry &roll_ );
 
-    rollImage( const int columns_, const int rows_ );
+    rollImage( const ssize_t columns_, const ssize_t rows_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    int _columns;
-    int _rows;
+    size_t _columns;
+    size_t _rows;
   };
 
   // Rotate image counter-clockwise by specified number of degrees.
@@ -1046,12 +1046,12 @@ namespace Magick
   class MagickDLLDecl spreadImage : public std::unary_function<Image&,void>
   {
   public:
-    spreadImage( const unsigned int amount_ = 3 );
+    spreadImage( const size_t amount_ = 3 );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _amount;
+    size_t _amount;
   };
 
   // Add a digital watermark to the image (based on second image)
@@ -1225,12 +1225,12 @@ namespace Magick
   class MagickDLLDecl animationDelayImage : public std::unary_function<Image&,void>
   {
   public:
-    animationDelayImage( const unsigned int delay_ );
+    animationDelayImage( const size_t delay_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _delay;
+    size_t _delay;
   };
 
   // Number of iterations to loop an animation (e.g. Netscape loop
@@ -1238,12 +1238,12 @@ namespace Magick
   class MagickDLLDecl animationIterationsImage : public std::unary_function<Image&,void>
   {
   public:
-    animationIterationsImage( const unsigned int iterations_ );
+    animationIterationsImage( const size_t iterations_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _iterations;
+    size_t _iterations;
   };
 
   // Image background color
@@ -1362,12 +1362,12 @@ namespace Magick
   class MagickDLLDecl colorMapImage : public std::unary_function<Image&,void>
   {
   public:
-    colorMapImage( const unsigned int index_, const Color &color_ );
+    colorMapImage( const size_t index_, const Color &color_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _index;
+    size_t _index;
     Color        _color;
   };
 
@@ -1412,12 +1412,12 @@ namespace Magick
   class MagickDLLDecl depthImage : public std::unary_function<Image&,void>
   {
   public:
-    depthImage( const unsigned int depth_ );
+    depthImage( const size_t depth_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _depth;
+    size_t _depth;
   };
 
   // Endianness (LSBEndian like Intel or MSBEndian like SPARC) for image
@@ -1473,24 +1473,24 @@ namespace Magick
   class MagickDLLDecl fontPointsizeImage : public std::unary_function<Image&,void>
   {
   public:
-    fontPointsizeImage( const unsigned int pointsize_ );
+    fontPointsizeImage( const size_t pointsize_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _pointsize;
+    size_t _pointsize;
   };
 
   // GIF disposal method
   class MagickDLLDecl gifDisposeMethodImage : public std::unary_function<Image&,void>
   {
   public:
-    gifDisposeMethodImage( const unsigned int disposeMethod_ );
+    gifDisposeMethodImage( const size_t disposeMethod_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _disposeMethod;
+    size_t _disposeMethod;
   };
 
   // Type of interlacing to use
@@ -1593,15 +1593,15 @@ namespace Magick
   class MagickDLLDecl pixelColorImage : public std::unary_function<Image&,void>
   {
   public:
-    pixelColorImage( const unsigned int x_,
-                     const unsigned int y_,
+    pixelColorImage( const ssize_t x_,
+                     const ssize_t y_,
          const Color &color_);
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _x;
-    unsigned int _y;
+    ssize_t _x;
+    ssize_t _y;
     Color        _color;
   };
 
@@ -1621,24 +1621,24 @@ namespace Magick
   class MagickDLLDecl qualityImage : public std::unary_function<Image&,void>
   {
   public:
-    qualityImage( const unsigned int quality_ );
+    qualityImage( const size_t quality_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _quality;
+    size_t _quality;
   };
 
   // Maximum number of colors to quantize to
   class MagickDLLDecl quantizeColorsImage : public std::unary_function<Image&,void>
   {
   public:
-    quantizeColorsImage( const unsigned int colors_ );
+    quantizeColorsImage( const size_t colors_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _colors;
+    size_t _colors;
   };
 
   // Colorspace to quantize in.
@@ -1669,12 +1669,12 @@ namespace Magick
   class MagickDLLDecl quantizeTreeDepthImage : public std::unary_function<Image&,void>
   {
   public:
-    quantizeTreeDepthImage( const unsigned int treeDepth_ );
+    quantizeTreeDepthImage( const size_t treeDepth_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _treeDepth;
+    size_t _treeDepth;
   };
 
   // The type of rendering intent
@@ -1705,26 +1705,26 @@ namespace Magick
   class MagickDLLDecl sceneImage : public std::unary_function<Image&,void>
   {
   public:
-    sceneImage( const unsigned int scene_ );
+    sceneImage( const size_t scene_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _scene;
+    size_t _scene;
   };
 
   // adjust the image contrast with a non-linear sigmoidal contrast algorithm
   class MagickDLLDecl sigmoidalContrastImage : public std::unary_function<Image&,void>
   {
   public:
-    sigmoidalContrastImage( const unsigned int sharpen_,
+    sigmoidalContrastImage( const size_t sharpen_,
       const double contrast,
       const double midpoint = QuantumRange / 2.0 );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _sharpen;
+    size_t _sharpen;
     double contrast;
     double midpoint;
   };
@@ -1745,24 +1745,24 @@ namespace Magick
   class MagickDLLDecl subImageImage : public std::unary_function<Image&,void>
   {
   public:
-    subImageImage( const unsigned int subImage_ );
+    subImageImage( const size_t subImage_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _subImage;
+    size_t _subImage;
   };
 
   // Number of images relative to the base image
   class MagickDLLDecl subRangeImage : public std::unary_function<Image&,void>
   {
   public:
-    subRangeImage( const unsigned int subRange_ );
+    subRangeImage( const size_t subRange_ );
 
     void operator()( Image &image_ ) const;
 
   private:
-    unsigned int _subRange;
+    size_t _subRange;
   };
 
   // Tile name
@@ -1846,7 +1846,7 @@ namespace Magick
        InputIterator last_ ) {
 
     MagickCore::Image* previous = 0;
-    int scene = 0;
+    ssize_t scene = 0;
     for ( InputIterator iter = first_; iter != last_; ++iter )
       {
   // Unless we reduce the reference count to one, the same image
@@ -2010,7 +2010,7 @@ namespace Magick
                       CoderInfo::MatchType isMultiFrame_ = CoderInfo::AnyMatch
                       ) {
     // Obtain first entry in MagickInfo list
-    unsigned long number_formats;
+    size_t number_formats;
     MagickCore::ExceptionInfo exceptionInfo;
     MagickCore::GetExceptionInfo( &exceptionInfo );
     char **coder_list =
@@ -2025,7 +2025,7 @@ namespace Magick
     // Clear out container
     container_->clear();
 
-    for ( int i=0; i < (long) number_formats; i++)
+    for ( ssize_t i=0; i < (ssize_t) number_formats; i++)
       {
         const MagickCore::MagickInfo *magick_info =
           MagickCore::GetMagickInfo( coder_list[i], &exceptionInfo );
@@ -2073,7 +2073,7 @@ namespace Magick
 
   //
   // Fill container with color histogram.
-  // Entries are of type "std::pair<Color,unsigned long>".  Use the pair
+  // Entries are of type "std::pair<Color,size_t>".  Use the pair
   // "first" member to access the Color and the "second" member to access
   // the number of times the color occurs in the image.
   //
@@ -2082,9 +2082,9 @@ namespace Magick
   //  Using <map>:
   //
   //  Image image("image.miff");
-  //  map<Color,unsigned long> histogram;
+  //  map<Color,size_t> histogram;
   //  colorHistogram( &histogram, image );
-  //  std::map<Color,unsigned long>::const_iterator p=histogram.begin();
+  //  std::map<Color,size_t>::const_iterator p=histogram.begin();
   //  while (p != histogram.end())
   //    {
   //      cout << setw(10) << (int)p->second << ": ("
@@ -2098,9 +2098,9 @@ namespace Magick
   //  Using <vector>:
   //
   //  Image image("image.miff");
-  //  std::vector<std::pair<Color,unsigned long> > histogram;
+  //  std::vector<std::pair<Color,size_t> > histogram;
   //  colorHistogram( &histogram, image );
-  //  std::vector<std::pair<Color,unsigned long> >::const_iterator p=histogram.begin();
+  //  std::vector<std::pair<Color,size_t> >::const_iterator p=histogram.begin();
   //  while (p != histogram.end())
   //    {
   //      cout << setw(10) << (int)p->second << ": ("
@@ -2118,7 +2118,7 @@ namespace Magick
     MagickCore::GetExceptionInfo( &exceptionInfo );
 
     // Obtain histogram array
-    unsigned long colors;
+    size_t colors;
     MagickCore::ColorPacket *histogram_array = 
       MagickCore::GetImageHistogram( image.constImage(), &colors, &exceptionInfo );
     throwException( exceptionInfo );
@@ -2128,13 +2128,13 @@ namespace Magick
     histogram_->clear();
 
     // Transfer histogram array to container
-    for ( unsigned long i=0; i < colors; i++)
+    for ( size_t i=0; i < colors; i++)
       {
-        histogram_->insert(histogram_->end(),std::pair<const Color,unsigned long>
+        histogram_->insert(histogram_->end(),std::pair<const Color,size_t>
                            ( Color(histogram_array[i].pixel.red,
                                    histogram_array[i].pixel.green,
                                    histogram_array[i].pixel.blue),
-                                   (unsigned long) histogram_array[i].count) );
+                                   (size_t) histogram_array[i].count) );
       }
     
     // Deallocate histogram array
@@ -2318,7 +2318,7 @@ namespace Magick
   void morphImages( Container *morphedImages_,
         InputIterator first_,
         InputIterator last_,
-        unsigned int frames_ ) {
+        size_t frames_ ) {
     MagickCore::ExceptionInfo exceptionInfo;
     MagickCore::GetExceptionInfo( &exceptionInfo );
 
@@ -2438,7 +2438,7 @@ namespace Magick
     MagickCore::GetExceptionInfo( &exceptionInfo );
 
     linkImages( first_, last_ );
-    int errorStat = MagickCore::WriteImages( first_->constImageInfo(),
+    ssize_t errorStat = MagickCore::WriteImages( first_->constImageInfo(),
                                             first_->image(),
                                             imageSpec_.c_str(),
                                             &exceptionInfo );
