@@ -370,8 +370,8 @@ static MagickBooleanType IsDPX(const unsigned char *magick,const size_t extent)
 %
 */
 
-static size_t GetBytesPerRow(unsigned long columns,
-  unsigned long samples_per_pixel,unsigned long bits_per_pixel,
+static size_t GetBytesPerRow(size_t columns,
+  size_t samples_per_pixel,size_t bits_per_pixel,
   MagickBooleanType pad)
 {
   size_t
@@ -513,14 +513,14 @@ static void SetPrimaryChromaticity(const DPXColorimetric colorimetric,
   }
 }
 
-static void TimeCodeToString(const unsigned long timestamp,char *code)
+static void TimeCodeToString(const size_t timestamp,char *code)
 {
 #define TimeFields  7
 
   unsigned int
     shift;
 
-  register long
+  register ssize_t
     i;
 
   *code='\0';
@@ -539,9 +539,6 @@ static void TimeCodeToString(const unsigned long timestamp,char *code)
 
 static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  CacheView
-    *image_view;
-
   char
     magick[4],
     value[MaxTextExtent];
@@ -552,7 +549,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
+  ssize_t
     row,
     y;
 
@@ -568,7 +565,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   QuantumType
     quantum_type;
 
-  register long
+  register ssize_t
     i;
 
   ssize_t
@@ -580,7 +577,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     component_type;
 
-  unsigned long
+  size_t
     samples_per_pixel;
 
   /*
@@ -903,18 +900,18 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       dpx.television.interlace=(unsigned char) ReadBlobByte(image);
       offset++;
       if (dpx.television.interlace != 0)
-        (void) FormatImageProperty(image,"dpx:television.interlace","%ld",(long)
-          dpx.television.interlace);
+        (void) FormatImageProperty(image,"dpx:television.interlace","%.20g",
+          (double) dpx.television.interlace);
       dpx.television.field_number=(unsigned char) ReadBlobByte(image);
       offset++;
       if (dpx.television.field_number != 0)
-        (void) FormatImageProperty(image,"dpx:television.field_number","%ld",
-          (long) dpx.television.field_number);
+        (void) FormatImageProperty(image,"dpx:television.field_number","%.20g",
+          (double) dpx.television.field_number);
       dpx.television.video_signal=(unsigned char) ReadBlobByte(image);
       offset++;
       if (dpx.television.video_signal != 0)
-        (void) FormatImageProperty(image,"dpx:television.video_signal","%ld",
-          (long) dpx.television.video_signal);
+        (void) FormatImageProperty(image,"dpx:television.video_signal","%.20g",
+          (double) dpx.television.video_signal);
       dpx.television.padding=(unsigned char) ReadBlobByte(image);
       offset++;
       if (dpx.television.padding != 0)
@@ -988,14 +985,14 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           StringInfo
             *profile;
 
-           profile=AcquireStringInfo(dpx.file.user_size-sizeof(dpx.user.id));
+           profile=AcquireStringInfo(dpx.file.user_size);
            offset+=ReadBlob(image,GetStringInfoLength(profile),
              GetStringInfoDatum(profile));
-           (void) SetImageProfile(image,"dpx:user.data",profile);
+           (void) SetImageProfile(image,"dpx",profile);
            profile=DestroyStringInfo(profile);
         }
     }
-  for ( ; offset < (long) dpx.file.image_offset; offset++)
+  for ( ; offset < (ssize_t) dpx.file.image_offset; offset++)
     (void) ReadBlobByte(image);
   /*
     Read DPX image header.
@@ -1079,13 +1076,9 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   SetQuantumQuantum(quantum_info,32);
   SetQuantumPack(quantum_info,dpx.image.image_element[0].packing == 0 ?
     MagickTrue : MagickFalse);
-  image_view=AcquireCacheView(image);
-#if defined(MAGICKCORE_OPENMP_SUPPORT) && (_OPENMP > 200505)
-  #pragma omp parallel for schedule(static) shared(row,status,quantum_type)
-#endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    long
+    ssize_t
       offset;
 
     MagickBooleanType
@@ -1106,9 +1099,6 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (status == MagickFalse)
       continue;
     pixels=GetQuantumPixels(quantum_info);
-#if defined(MAGICKCORE_OPENMP_SUPPORT) && (_OPENMP > 200505)
-  #pragma omp critical (MagickCore_ReadDPXImage)
-#endif
     {
       count=ReadBlob(image,extent,pixels);
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -1117,7 +1107,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           MagickBooleanType
             proceed;
 
-          proceed=SetImageProgress(image,LoadImageTag,row,image->rows);
+          proceed=SetImageProgress(image,LoadImageTag,(MagickOffsetType) row,
+                image->rows);
           if (proceed == MagickFalse)
             status=MagickFalse;
         }
@@ -1125,20 +1116,18 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (count != (ssize_t) extent)
       status=MagickFalse;
-    q=QueueCacheViewAuthenticPixels(image_view,0,offset,image->columns,1,
-      exception);
+    q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    length=ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
-      pixels,exception);
-    sync=SyncCacheViewAuthenticPixels(image_view,exception);
+    length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+      quantum_type,pixels,exception);
+    sync=SyncAuthenticPixels(image,exception);
     if (sync == MagickFalse)
       status=MagickFalse;
   }
-  image_view=DestroyCacheView(image_view);
   quantum_info=DestroyQuantumInfo(quantum_info);
   if (status == MagickFalse)
     ThrowReaderException(CorruptImageError,"UnableToReadImageData");
@@ -1170,10 +1159,10 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterDPXImage method is:
 %
-%      unsigned long RegisterDPXImage(void)
+%      size_t RegisterDPXImage(void)
 %
 */
-ModuleExport unsigned long RegisterDPXImage(void)
+ModuleExport size_t RegisterDPXImage(void)
 {
   MagickInfo
     *entry;
@@ -1263,7 +1252,7 @@ static unsigned int StringToTimeCode(const char *key)
   char
     buffer[2];
 
-  register long
+  register ssize_t
     i;
 
   unsigned int
@@ -1298,7 +1287,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   DPXInfo
     dpx;
 
-  long
+  ssize_t
     horizontal_factor,
     vertical_factor,
     y;
@@ -1324,7 +1313,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -1359,8 +1348,8 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
+      horizontal_factor=(ssize_t) geometry_info.rho;
+      vertical_factor=(ssize_t) geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
@@ -1383,9 +1372,11 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   dpx.file.magic=0x53445058U;
   offset+=WriteBlobLong(image,dpx.file.magic);
   dpx.file.image_offset=0x2000U;
-  profile=GetImageProfile(image,"dpx:user.data");
+  profile=GetImageProfile(image,"dpx");
   if (profile != (StringInfo *) NULL)
     {
+      if (GetStringInfoLength(profile) > 1048576)
+        ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
       dpx.file.image_offset+=(unsigned int) GetStringInfoLength(profile);
       dpx.file.image_offset=(((dpx.file.image_offset+0x2000-1)/0x2000)*0x2000);
     }
@@ -1418,7 +1409,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     dpx.file.timestamp);
   offset+=WriteBlob(image,sizeof(dpx.file.timestamp),(unsigned char *)
     dpx.file.timestamp);
-  (void) strncpy(dpx.file.creator,GetMagickVersion((unsigned long *) NULL),
+  (void) strncpy(dpx.file.creator,GetMagickVersion((size_t *) NULL),
     sizeof(dpx.file.creator));
   value=GetDPXProperty(image_info,image,"dpx:file.creator");
   if (value != (const char *) NULL)
@@ -1819,7 +1810,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
       extent=GetBytesPerRow(image->columns,1UL,image->depth,MagickTrue);
     }
   pixels=GetQuantumPixels(quantum_info);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
@@ -1829,7 +1820,8 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     count=WriteBlob(image,extent,pixels);
     if (count != (ssize_t) extent)
       break;
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
     if (status == MagickFalse)
       break;
   }
