@@ -178,7 +178,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register char
     *p;
 
-  register long
+  register ssize_t
     c;
 
   SegmentInfo
@@ -187,7 +187,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ssize_t
     count;
 
-  unsigned long
+  size_t
     height,
     width;
 
@@ -290,8 +290,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Set PCL render geometry.
     */
-    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+    width=(size_t) floor(bounds.x2-bounds.x1+0.5);
+    height=(size_t) floor(bounds.y2-bounds.y1+0.5);
     if (width > page.width)
       page.width=width;
     if (height > page.height)
@@ -305,8 +305,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
-  (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
-    page.width,page.height);
+  (void) FormatMagickString(geometry,MaxTextExtent,"%.20gx%.20g",(double)
+    page.width,(double) page.height);
   if (image_info->monochrome != MagickFalse)
     delegate_info=GetDelegateInfo("pcl:mono",(char *) NULL,exception);
   else
@@ -323,22 +323,23 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
     image->x_resolution,image->y_resolution);
-  page.width=(unsigned long) (page.width*image->x_resolution/delta.x+0.5);
-  page.height=(unsigned long) (page.height*image->y_resolution/delta.y+0.5);
-  (void) FormatMagickString(options,MaxTextExtent,"-g%lux%lu ",
-    page.width,page.height);
+  page.width=(size_t) floor(page.width*image->x_resolution/delta.x+0.5);
+  page.height=(size_t) floor(page.height*image->y_resolution/delta.y+
+    0.5);
+  (void) FormatMagickString(options,MaxTextExtent,"-g%.20gx%.20g ",(double)
+     page.width,(double) page.height);
   image=DestroyImage(image);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
     {
       if (read_info->number_scenes != 1)
-        (void) FormatMagickString(options,MaxTextExtent,"-dLastPage=%lu",
-          read_info->scene+read_info->number_scenes);
+        (void) FormatMagickString(options,MaxTextExtent,"-dLastPage=%.20g",
+          (double) (read_info->scene+read_info->number_scenes));
       else
         (void) FormatMagickString(options,MaxTextExtent,
-          "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,read_info->scene+
-          read_info->number_scenes);
+          "-dFirstPage=%.20g -dLastPage=%.20g",(double) read_info->scene+1,
+          (double) (read_info->scene+read_info->number_scenes));
       read_info->number_scenes=0;
       if (read_info->scenes != (char *) NULL)
         *read_info->scenes='\0';
@@ -404,10 +405,10 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPCLImage method is:
 %
-%      unsigned long RegisterPCLImage(void)
+%      size_t RegisterPCLImage(void)
 %
 */
-ModuleExport unsigned long RegisterPCLImage(void)
+ModuleExport size_t RegisterPCLImage(void)
 {
   MagickInfo
     *entry;
@@ -484,7 +485,7 @@ static size_t PCLDeltaCompressImage(const size_t length,
     j,
     replacement;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -492,10 +493,10 @@ static size_t PCLDeltaCompressImage(const size_t length,
     *q;
 
   q=compress_pixels;
-  for (x=0; x < (long) length; )
+  for (x=0; x < (ssize_t) length; )
   {
     j=0;
-    for (i=0; x < (long) length; x++)
+    for (i=0; x < (ssize_t) length; x++)
     {
       if (*pixels++ != *previous_pixels++)
         {
@@ -504,7 +505,7 @@ static size_t PCLDeltaCompressImage(const size_t length,
         }
       j++;
     }
-    for ( ; x < (long) length; x++)
+    for ( ; x < (ssize_t) length; x++)
     {
       if (*pixels == *previous_pixels)
         break;
@@ -551,10 +552,10 @@ static size_t PCLPackbitsCompressImage(const size_t length,
   int
     count;
 
-  long
+  ssize_t
     j;
 
-  register long
+  register ssize_t
     x;
 
   register unsigned char
@@ -567,7 +568,7 @@ static size_t PCLPackbitsCompressImage(const size_t length,
     Compress pixels with Packbits encoding.
   */
   q=compress_pixels;
-  for (x=(long) length; x != 0; )
+  for (x=(ssize_t) length; x != 0; )
   {
     switch (x)
     {
@@ -609,7 +610,7 @@ static size_t PCLPackbitsCompressImage(const size_t length,
               Packed run.
             */
             count=3;
-            while (((long) count < x) && (*pixels == *(pixels+count)))
+            while (((ssize_t) count < x) && (*pixels == *(pixels+count)))
             {
               count++;
               if (count >= 127)
@@ -630,12 +631,12 @@ static size_t PCLPackbitsCompressImage(const size_t length,
         {
           packbits[count+1]=pixels[count];
           count++;
-          if (((long) count >= (x-3)) || (count >= 127))
+          if (((ssize_t) count >= (x-3)) || (count >= 127))
             break;
         }
         x-=count;
         *packbits=(unsigned char) (count-1);
-        for (j=0; j <= (long) count; j++)
+        for (j=0; j <= (ssize_t) count; j++)
           *q++=packbits[j];
         pixels+=count;
         break;
@@ -654,7 +655,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   const char
     *option;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -669,7 +670,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -686,8 +687,9 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
     *pixels,
     *previous_pixels;
 
-  unsigned long
-    density;
+  size_t
+    density,
+    one;
 
   /*
     Open output image file.
@@ -708,9 +710,10 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         geometry;
 
       (void) ParseGeometry(image_info->density,&geometry);
-      density=(unsigned long) geometry.rho;
+      density=(size_t) geometry.rho;
     }
   scene=0;
+  one=1;
   do
   {
     if (image->colorspace != RGBColorspace)
@@ -720,10 +723,11 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
     */
     (void) WriteBlobString(image,"\033E");  /* printer reset */
     (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
-    (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%lus%luT",
-      image->columns,image->rows);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%.20gs%.20gT",
+      (double) image->columns,(double) image->rows);
     (void) WriteBlobString(image,buffer);
-    (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%.20gR",(double)
+      density);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
     if (IsMonochromeImage(image,&image->exception) != MagickFalse)
@@ -772,17 +776,20 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobByte(image,8); /* bits per red component */
           (void) WriteBlobByte(image,8); /* bits per green component */
           (void) WriteBlobByte(image,8); /* bits per blue component */
-          for (i=0; i < (long) image->colors; i++)
+          for (i=0; i < (ssize_t) image->colors; i++)
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "\033*v%da%db%dc%ldI",ScaleQuantumToChar(image->colormap[i].red),
+              "\033*v%da%db%dc%.20gI",
+              ScaleQuantumToChar(image->colormap[i].red),
               ScaleQuantumToChar(image->colormap[i].green),
-              ScaleQuantumToChar(image->colormap[i].blue),i);
+              ScaleQuantumToChar(image->colormap[i].blue),(double) i);
             (void) WriteBlobString(image,buffer);
           }
-          for ( ; i < (1L << bits_per_pixel); i++)
+          one=1;
+          for ( ; i < (ssize_t) (one << bits_per_pixel); i++)
           {
-            (void) FormatMagickString(buffer,MaxTextExtent,"\033*v%luI",i);
+            (void) FormatMagickString(buffer,MaxTextExtent,"\033*v%.20gI",
+              (double) i);
             (void) WriteBlobString(image,buffer);
           }
         }
@@ -843,7 +850,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         break;
       }
     }
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
@@ -863,7 +870,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           */
           bit=0;
           byte=0;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             byte<<=1;
             if (PixelIntensity(p) >= ((MagickRealType) QuantumRange/2.0))
@@ -886,7 +893,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           /*
             Colormapped image.
           */
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
             *q++=(unsigned char) indexes[x];
           break;
         }
@@ -896,7 +903,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           /*
             Truecolor image.
           */
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
             *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
@@ -910,8 +917,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       {
         case NoCompression:
         {
-          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-            (unsigned long) length);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) length);
           (void) WriteBlobString(image,buffer);
           (void) WriteBlob(image,length,pixels);
           break;
@@ -920,8 +927,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         {
           packets=PCLPackbitsCompressImage(length,pixels,
             compress_pixels);
-          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-            (unsigned long) packets);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) packets);
           (void) WriteBlobString(image,buffer);
           (void) WriteBlob(image,packets,compress_pixels);
           break;
@@ -929,12 +936,12 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         default:
         {
           if (y == 0)
-            for (i=0; i < (long) length; i++)
+            for (i=0; i < (ssize_t) length; i++)
               previous_pixels[i]=(~pixels[i]);
           packets=PCLDeltaCompressImage(length,previous_pixels,pixels,
             compress_pixels);
-          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-            (unsigned long) packets);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) packets);
           (void) WriteBlobString(image,buffer);
           (void) WriteBlob(image,packets,compress_pixels);
           (void) CopyMagickMemory(previous_pixels,pixels,length*
