@@ -78,11 +78,11 @@ typedef struct _AESInfo
     *encipher_key,
     *decipher_key;
 
-  long
+  ssize_t
     rounds,
     timestamp;
 
-  unsigned long
+  size_t
     signature;
 } AESInfo;
 
@@ -204,7 +204,7 @@ static AESInfo *AcquireAESInfo(void)
       (aes_info->encipher_key == (unsigned int *) NULL) ||
       (aes_info->decipher_key == (unsigned int *) NULL))
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  aes_info->timestamp=(long) time(0);
+  aes_info->timestamp=(ssize_t) time(0);
   aes_info->signature=MagickSignature;
   return(aes_info);
 }
@@ -281,7 +281,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 static inline void AddRoundKey(const unsigned int *ciphertext,
   const unsigned int *key,unsigned int *plaintext)
 {
-  register long
+  register ssize_t
     i;
 
   /*
@@ -380,7 +380,7 @@ static inline unsigned int RotateLeft(const unsigned int x)
 static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
   unsigned char *ciphertext)
 {
-  register long
+  register ssize_t
     i,
     j;
 
@@ -562,7 +562,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   IndexPacket
     *indexes;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -645,9 +645,9 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -660,7 +660,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     p=pixels;
-    for (x=0; x < (long) length; x++)
+    for (x=0; x < (ssize_t) length; x++)
     {
       (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
@@ -674,7 +674,8 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
       pixels,exception);
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
-    proceed=SetImageProgress(image,DecipherImageTag,y,image->rows);
+    proceed=SetImageProgress(image,DecipherImageTag,(MagickOffsetType) y,
+      image->rows);
     if (proceed == MagickFalse)
       break;
   }
@@ -690,7 +691,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   aes_info=DestroyAESInfo(aes_info);
   (void) ResetMagickMemory(input_block,0,sizeof(input_block));
   (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -764,7 +765,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   IndexPacket
     *indexes;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -854,9 +855,9 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -869,7 +870,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     p=pixels;
-    for (x=0; x < (long) length; x++)
+    for (x=0; x < (ssize_t) length; x++)
     {
       (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
@@ -883,7 +884,8 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
       pixels,exception);
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
-    proceed=SetImageProgress(image,EncipherImageTag,y,image->rows);
+    proceed=SetImageProgress(image,EncipherImageTag,(MagickOffsetType) y,
+      image->rows);
     if (proceed == MagickFalse)
       break;
   }
@@ -896,7 +898,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   aes_info=DestroyAESInfo(aes_info);
   (void) ResetMagickMemory(input_block,0,sizeof(input_block));
   (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -962,11 +964,11 @@ static inline unsigned int RotateRight(const unsigned int x)
 
 static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 {
-  long
+  ssize_t
     bytes,
     n;
 
-  register long
+  register ssize_t
     i;
 
   unsigned char
