@@ -81,7 +81,7 @@ typedef struct _NodeInfo
   MagickSizeType
     number_unique;
 
-  unsigned long
+  size_t
     level;
 } NodeInfo;
 
@@ -99,11 +99,13 @@ typedef struct _CubeInfo
   NodeInfo
     *root;
 
-  long
-    x,
+  ssize_t
+    x;
+
+  MagickOffsetType
     progress;
 
-  unsigned long
+  size_t
     colors,
     free_nodes;
 
@@ -121,7 +123,7 @@ static CubeInfo
   *GetCubeInfo(void);
 
 static NodeInfo
-  *GetNodeInfo(CubeInfo *,const unsigned long);
+  *GetNodeInfo(CubeInfo *,const size_t);
 
 static void
   DestroyColorCube(const Image *,NodeInfo *);
@@ -139,7 +141,7 @@ static void
 %
 %  ClassifyImageColors() builds a populated CubeInfo tree for the specified
 %  image.  The returned tree should be deallocated using DestroyCubeInfo()
-%  once it is no longer needed.
+%  once it is no ssize_ter needed.
 %
 %  The format of the ClassifyImageColors() method is:
 %
@@ -154,13 +156,13 @@ static void
 %
 */
 
-static inline unsigned long ColorToNodeId(const Image *image,
-  const MagickPixelPacket *pixel,unsigned long index)
+static inline size_t ColorToNodeId(const Image *image,
+  const MagickPixelPacket *pixel,size_t index)
 {
-  unsigned long
+  size_t
     id;
 
-  id=(unsigned long) (
+  id=(size_t) (
     ((ScaleQuantumToChar(ClampToQuantum(pixel->red)) >> index) & 0x01) |
     ((ScaleQuantumToChar(ClampToQuantum(pixel->green)) >> index) & 0x01) << 1 |
     ((ScaleQuantumToChar(ClampToQuantum(pixel->blue)) >> index) & 0x01) << 2);
@@ -181,7 +183,7 @@ static CubeInfo *ClassifyImageColors(const Image *image,
   CubeInfo
     *cube_info;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -200,11 +202,11 @@ static CubeInfo *ClassifyImageColors(const Image *image,
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i,
     x;
 
-  register unsigned long
+  register size_t
     id,
     index,
     level;
@@ -226,13 +228,13 @@ static CubeInfo *ClassifyImageColors(const Image *image,
   GetMagickPixelPacket(image,&pixel);
   GetMagickPixelPacket(image,&target);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       /*
         Start at the root and proceed level by level.
@@ -257,14 +259,14 @@ static CubeInfo *ClassifyImageColors(const Image *image,
         node_info=node_info->child[id];
         index--;
       }
-      for (i=0; i < (long) node_info->number_unique; i++)
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
         if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
           break;
       }
-      if (i < (long) node_info->number_unique)
+      if (i < (ssize_t) node_info->number_unique)
         node_info->list[i].count++;
       else
         {
@@ -291,7 +293,8 @@ static CubeInfo *ClassifyImageColors(const Image *image,
         }
       p++;
     }
-    proceed=SetImageProgress(image,EvaluateImageTag,y,image->rows);
+    proceed=SetImageProgress(image,EvaluateImageTag,(MagickOffsetType) y,
+      image->rows);
     if (proceed == MagickFalse)
       break;
   }
@@ -332,17 +335,17 @@ static CubeInfo *ClassifyImageColors(const Image *image,
 static void DefineImageHistogram(const Image *image,NodeInfo *node_info,
   ColorPacket **histogram)
 {
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_children;
 
   /*
     Traverse any children.
   */
   number_children=image->matte == MagickFalse ? 8UL : 16UL;
-  for (i=0; i < (long) number_children; i++)
+  for (i=0; i < (ssize_t) number_children; i++)
     if (node_info->child[i] != (NodeInfo *) NULL)
       DefineImageHistogram(image,node_info->child[i],histogram);
   if (node_info->level == (MaxTreeDepth-1))
@@ -351,7 +354,7 @@ static void DefineImageHistogram(const Image *image,NodeInfo *node_info,
         *p;
 
       p=node_info->list;
-      for (i=0; i < (long) node_info->number_unique; i++)
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
       {
         (*histogram)->pixel=p->pixel;
         (*histogram)->index=p->index;
@@ -433,17 +436,17 @@ static CubeInfo *DestroyCubeInfo(const Image *image,CubeInfo *cube_info)
 */
 static void DestroyColorCube(const Image *image,NodeInfo *node_info)
 {
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_children;
 
   /*
     Traverse any children.
   */
   number_children=image->matte == MagickFalse ? 8UL : 16UL;
-  for (i=0; i < (long) number_children; i++)
+  for (i=0; i < (ssize_t) number_children; i++)
     if (node_info->child[i] != (NodeInfo *) NULL)
       DestroyColorCube(image,node_info->child[i]);
   if (node_info->list != (ColorPacket *) NULL)
@@ -508,8 +511,8 @@ static CubeInfo *GetCubeInfo(void)
 %
 %  The format of the GetImageHistogram method is:
 %
-%      unsigned long GetImageHistogram(const Image *image,
-%        unsigned long *number_colors,ExceptionInfo *exception)
+%      size_t GetImageHistogram(const Image *image,
+%        size_t *number_colors,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -521,7 +524,7 @@ static CubeInfo *GetCubeInfo(void)
 %
 */
 MagickExport ColorPacket *GetImageHistogram(const Image *image,
-  unsigned long *number_colors,ExceptionInfo *exception)
+  size_t *number_colors,ExceptionInfo *exception)
 {
   ColorPacket
     *histogram;
@@ -569,7 +572,7 @@ MagickExport ColorPacket *GetImageHistogram(const Image *image,
 %
 %  The format of the GetNodeInfo method is:
 %
-%      NodeInfo *GetNodeInfo(CubeInfo *cube_info,const unsigned long level)
+%      NodeInfo *GetNodeInfo(CubeInfo *cube_info,const size_t level)
 %
 %  A description of each parameter follows.
 %
@@ -578,7 +581,7 @@ MagickExport ColorPacket *GetImageHistogram(const Image *image,
 %    o level: Specifies the level in the storage_class the node resides.
 %
 */
-static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const unsigned long level)
+static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const size_t level)
 {
   NodeInfo
     *node_info;
@@ -643,7 +646,7 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
   CubeInfo
     *cube_info;
 
-  long
+  ssize_t
     y;
 
   MagickPixelPacket
@@ -656,16 +659,16 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
   register NodeInfo
     *node_info;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     id,
     index,
     level;
@@ -691,13 +694,13 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
   GetMagickPixelPacket(image,&pixel);
   GetMagickPixelPacket(image,&target);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       /*
         Start at the root and proceed level by level.
@@ -724,14 +727,14 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
       }
       if (level < MaxTreeDepth)
         break;
-      for (i=0; i < (long) node_info->number_unique; i++)
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
         if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
           break;
       }
-      if (i < (long) node_info->number_unique)
+      if (i < (ssize_t) node_info->number_unique)
         node_info->list[i].count++;
       else
         {
@@ -763,12 +766,12 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
         }
       p++;
     }
-    if (x < (long) image->columns)
+    if (x < (ssize_t) image->columns)
       break;
   }
   image_view=DestroyCacheView(image_view);
   cube_info=DestroyCubeInfo(image,cube_info);
-  return(y < (long) image->rows ? MagickFalse : MagickTrue);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -806,7 +809,7 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
   CubeInfo
     *cube_info;
 
-  long
+  ssize_t
     y;
 
   MagickPixelPacket
@@ -819,16 +822,16 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
   register NodeInfo
     *node_info;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     id,
     index,
     level;
@@ -854,13 +857,13 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
   GetMagickPixelPacket(image,&pixel);
   GetMagickPixelPacket(image,&target);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       /*
         Start at the root and proceed level by level.
@@ -887,14 +890,14 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
       }
       if (level < MaxTreeDepth)
         break;
-      for (i=0; i < (long) node_info->number_unique; i++)
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
         if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
           break;
       }
-      if (i < (long) node_info->number_unique)
+      if (i < (ssize_t) node_info->number_unique)
         node_info->list[i].count++;
       else
         {
@@ -926,12 +929,12 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
         }
       p++;
     }
-    if (x < (long) image->columns)
+    if (x < (ssize_t) image->columns)
       break;
   }
   image_view=DestroyCacheView(image_view);
   cube_info=DestroyCubeInfo(image,cube_info);
-  return(y < (long) image->rows ? MagickFalse : MagickTrue);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -959,6 +962,9 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
 %  will be extracted from all the given channels, and those channels will be
 %  stretched by exactly the same amount (preventing color distortion).
 %
+%  In the special case that only ONE value is found in a channel of the image
+%  that value is not stretched, that value is left as is.
+%
 %  The 'SyncChannels' is turned on in the 'DefaultChannels' setting by
 %  default.
 %
@@ -979,15 +985,18 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
 %      from the minimum and maximum points by this color value.
 %
 */
+
 MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
   const ChannelType channel,const double black_value,const double white_value)
 {
   double
-    min,max;
+    min,
+    max;
 
   MagickStatusType
     status;
 
+  status=MagickTrue;
   if ((channel & SyncChannels) != 0)
     {
       /*
@@ -996,18 +1005,20 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
       (void) GetImageChannelRange(image,channel,&min,&max,&image->exception);
       min+=black_value;
       max-=white_value;
-      return(LevelImageChannel(image,channel,min,max,1.0));
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,channel,min,max,1.0);
+      return(status != 0 ? MagickTrue : MagickFalse);
     }
   /*
     Auto-level each channel separately.
   */
-  status=MagickTrue;
   if ((channel & RedChannel) != 0)
     {
       (void) GetImageChannelRange(image,RedChannel,&min,&max,&image->exception);
       min+=black_value;
       max-=white_value;
-      status&=LevelImageChannel(image,RedChannel,min,max,1.0);
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,RedChannel,min,max,1.0);
     }
   if ((channel & GreenChannel) != 0)
     {
@@ -1015,7 +1026,8 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
         &image->exception);
       min+=black_value;
       max-=white_value;
-      status&=LevelImageChannel(image,GreenChannel,min,max,1.0);
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,GreenChannel,min,max,1.0);
     }
   if ((channel & BlueChannel) != 0)
     {
@@ -1023,7 +1035,8 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
         &image->exception);
       min+=black_value;
       max-=white_value;
-      status&=LevelImageChannel(image,BlueChannel,min,max,1.0);
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,BlueChannel,min,max,1.0);
     }
   if (((channel & OpacityChannel) != 0) &&
        (image->matte == MagickTrue))
@@ -1032,7 +1045,8 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
         &image->exception);
       min+=black_value;
       max-=white_value;
-      status&=LevelImageChannel(image,OpacityChannel,min,max,1.0);
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,OpacityChannel,min,max,1.0);
     }
   if (((channel & IndexChannel) != 0) &&
        (image->colorspace == CMYKColorspace))
@@ -1041,7 +1055,8 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
         &image->exception);
       min+=black_value;
       max-=white_value;
-      status&=LevelImageChannel(image,IndexChannel,min,max,1.0);
+      if (fabs(min-max) >= MagickEpsilon)
+        status&=LevelImageChannel(image,IndexChannel,min,max,1.0);
     }
   return(status != 0 ? MagickTrue : MagickFalse);
 }
@@ -1061,7 +1076,7 @@ MagickExport MagickBooleanType MinMaxStretchImage(Image *image,
 %
 %  The format of the GetNumberColors method is:
 %
-%      unsigned long GetNumberColors(const Image *image,FILE *file,
+%      size_t GetNumberColors(const Image *image,FILE *file,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
@@ -1099,7 +1114,7 @@ static int HistogramCompare(const void *x,const void *y)
 }
 #endif
 
-MagickExport unsigned long GetNumberColors(const Image *image,FILE *file,
+MagickExport size_t GetNumberColors(const Image *image,FILE *file,
   ExceptionInfo *exception)
 {
 #define HistogramImageTag  "Histogram/Image"
@@ -1121,10 +1136,10 @@ MagickExport unsigned long GetNumberColors(const Image *image,FILE *file,
   register ColorPacket
     *p;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_colors;
 
   number_colors=0;
@@ -1146,7 +1161,7 @@ MagickExport unsigned long GetNumberColors(const Image *image,FILE *file,
     HistogramCompare);
   GetMagickPixelPacket(image,&pixel);
   p=histogram;
-  for (i=0; i < (long) number_colors; i++)
+  for (i=0; i < (ssize_t) number_colors; i++)
   {
     SetMagickPixelPacket(image,&p->pixel,&p->index,&pixel);
     (void) CopyMagickString(tuple,"(",MaxTextExtent);
@@ -1168,14 +1183,15 @@ MagickExport unsigned long GetNumberColors(const Image *image,FILE *file,
     (void) ConcatenateMagickString(tuple,")",MaxTextExtent);
     (void) QueryMagickColorname(image,&pixel,SVGCompliance,color,exception);
     GetColorTuple(&pixel,MagickTrue,hex);
-    (void) fprintf(file,MagickSizeFormat,p->count);
+    (void) fprintf(file,"%10" MagickSizeFormat,p->count);
     (void) fprintf(file,": %s %s %s\n",tuple,hex,color);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        proceed=SetImageProgress(image,HistogramImageTag,i,number_colors);
+        proceed=SetImageProgress(image,HistogramImageTag,(MagickOffsetType) i,
+          number_colors);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1219,17 +1235,17 @@ static void UniqueColorsToImage(Image *image,CubeInfo *cube_info,
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_children;
 
   /*
     Traverse any children.
   */
   number_children=image->matte == MagickFalse ? 8UL : 16UL;
-  for (i=0; i < (long) number_children; i++)
+  for (i=0; i < (ssize_t) number_children; i++)
     if (node_info->child[i] != (NodeInfo *) NULL)
       UniqueColorsToImage(image,cube_info,node_info->child[i],exception);
   if (node_info->level == (MaxTreeDepth-1))
@@ -1244,7 +1260,7 @@ static void UniqueColorsToImage(Image *image,CubeInfo *cube_info,
         *restrict q;
 
       p=node_info->list;
-      for (i=0; i < (long) node_info->number_unique; i++)
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
       {
         q=QueueAuthenticPixels(image,cube_info->x,0,1,1,exception);
         if (q == (PixelPacket *) NULL)
