@@ -261,7 +261,7 @@ MagickExport ModuleInfo *GetModuleInfo(const char *tag,ExceptionInfo *exception)
 %  The format of the GetModuleInfoList function is:
 %
 %      const ModuleInfo **GetModuleInfoList(const char *pattern,
-%        unsigned long *number_modules,ExceptionInfo *exception)
+%        size_t *number_modules,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -295,7 +295,7 @@ static int ModuleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
-  unsigned long *number_modules,ExceptionInfo *exception)
+  size_t *number_modules,ExceptionInfo *exception)
 {
   const ModuleInfo
     **modules;
@@ -303,7 +303,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   register const ModuleInfo
     *p;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -311,7 +311,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_modules != (unsigned long *) NULL);
+  assert(number_modules != (size_t *) NULL);
   *number_modules=0;
   p=GetModuleInfo("*",exception);
   if (p == (const ModuleInfo *) NULL)
@@ -336,7 +336,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   UnlockSemaphoreInfo(module_semaphore);
   qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleInfoCompare);
   modules[i]=(ModuleInfo *) NULL;
-  *number_modules=(unsigned long) i;
+  *number_modules=(size_t) i;
   return(modules);
 }
 
@@ -356,7 +356,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %
 %  The format of the GetModuleList function is:
 %
-%      char **GetModuleList(const char *pattern,unsigned long *number_modules,
+%      char **GetModuleList(const char *pattern,size_t *number_modules,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -403,7 +403,7 @@ static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
 }
 
 MagickExport char **GetModuleList(const char *pattern,
-  unsigned long *number_modules,ExceptionInfo *exception)
+  size_t *number_modules,ExceptionInfo *exception)
 {
   char
     **modules,
@@ -417,7 +417,7 @@ MagickExport char **GetModuleList(const char *pattern,
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -427,7 +427,7 @@ MagickExport char **GetModuleList(const char *pattern,
     *buffer,
     *entry;
 
-  unsigned long
+  size_t
     max_entries;
 
   /*
@@ -463,7 +463,7 @@ MagickExport char **GetModuleList(const char *pattern,
       continue;
     if (GlobExpression(entry->d_name,pattern,MagickFalse) == MagickFalse)
       continue;
-    if (i >= (long) max_entries)
+    if (i >= (ssize_t) max_entries)
       {
         modules=(char **) NULL;
         if (~max_entries > max_entries)
@@ -496,7 +496,7 @@ MagickExport char **GetModuleList(const char *pattern,
     }
   qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleCompare);
   modules[i]=(char *) NULL;
-  *number_modules=(unsigned long) i;
+  *number_modules=(size_t) i;
   return(modules);
 }
 
@@ -628,7 +628,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       return(MagickTrue);
     }
 #else
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
     {
       const char
         *registery_key;
@@ -673,8 +673,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     }
 #endif
 #endif
-#if !defined(MAGICKCORE_CODER_PATH) && !defined(__WINDOWS__)
-# error MAGICKCORE_CODER_PATH or __WINDOWS__ must be defined when MAGICKCORE_INSTALLED_SUPPORT is defined
+#if !defined(MAGICKCORE_CODER_PATH) && !defined(MAGICKCORE_WINDOWS_SUPPORT)
+# error MAGICKCORE_CODER_PATH or MAGICKCORE_WINDOWS_SUPPORT must be defined when MAGICKCORE_INSTALLED_SUPPORT is defined
 #endif
 #else
   {
@@ -754,7 +754,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       if (IsPathAccessible(path) != MagickFalse)
         return(MagickTrue);
     }
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
     /*
       Search module path.
@@ -988,13 +988,14 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
   /*
     Execute the module.
   */
+  ClearMagickException(exception);
   image_filter=(ImageFilterHandler *) lt_dlsym(handle,name);
   if (image_filter == (ImageFilterHandler *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
       "UnableToLoadModule","`%s': %s",name,lt_dlerror());
   else
     {
-      unsigned long
+      size_t
         signature;
 
       if ((*images)->debug != MagickFalse)
@@ -1005,23 +1006,17 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),"\"%s\" completes",
           tag);
       if (signature != MagickImageFilterSignature)
-        {
-          (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-            "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,signature,
-            MagickImageFilterSignature);
-          return(MagickFalse);
-        }
+        (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
+          "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,
+          (unsigned long) signature,(unsigned long) MagickImageFilterSignature);
     }
   /*
     Close the module.
   */
   if (lt_dlclose(handle) != 0)
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
-        "UnableToCloseModule","`%s': %s",name,lt_dlerror());
-      return(MagickFalse);
-    }
-  return(MagickTrue);
+    (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
+      "UnableToCloseModule","`%s': %s",name,lt_dlerror());
+  return(exception->severity < ErrorException ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1054,10 +1049,10 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
   const ModuleInfo
     **module_info;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_modules;
 
   if (file == (const FILE *) NULL)
@@ -1076,7 +1071,7 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
   (void) fprintf(file,"Module\n");
   (void) fprintf(file,"-------------------------------------------------"
     "------------------------------\n");
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
   {
     if (module_info[i]->stealth != MagickFalse)
       continue;
@@ -1112,11 +1107,14 @@ MagickExport MagickBooleanType ModuleComponentGenesis(void)
   ExceptionInfo
     *exception;
 
+  MagickBooleanType
+    status;
+
   AcquireSemaphoreInfo(&module_semaphore);
   exception=AcquireExceptionInfo();
-  InitializeModuleList(exception);
+  status=InitializeModuleList(exception);
   exception=DestroyExceptionInfo(exception);
-  return(MagickTrue);
+  return(status);
 }
 
 /*
@@ -1191,7 +1189,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   size_t
     length;
 
-  unsigned long
+  size_t
     signature;
 
   /*
@@ -1241,8 +1239,8 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     Define RegisterFORMATImage method.
   */
   TagToModuleName(module_name,"Register%sImage",name);
-  module_info->register_module=(unsigned long (*)(void)) lt_dlsym(handle,name);
-  if (module_info->register_module == (unsigned long (*)(void)) NULL)
+  module_info->register_module=(size_t (*)(void)) lt_dlsym(handle,name);
+  if (module_info->register_module == (size_t (*)(void)) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
@@ -1270,7 +1268,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "ImageCoderSignatureMismatch","`%s': %8lx != %8lx",module_name,
-        signature,MagickImageCoderSignature);
+        (unsigned long) signature,(unsigned long) MagickImageCoderSignature);
       return(MagickFalse);
     }
   return(MagickTrue);
@@ -1303,10 +1301,10 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   char
     **modules;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_modules;
 
   /*
@@ -1317,12 +1315,12 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   modules=GetModuleList("*",&number_modules,exception);
   if (modules == (char **) NULL)
     return(MagickFalse);
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
     (void) OpenModule(modules[i],exception);
   /*
     Relinquish resources.
   */
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
     modules[i]=DestroyString(modules[i]);
   modules=(char **) RelinquishMagickMemory(modules);
   return(MagickTrue);
@@ -1408,7 +1406,7 @@ static void TagToCoderModuleName(const char *tag,char *name)
   (void) FormatMagickString(name,MaxTextExtent,"%s.la",tag);
   (void) LocaleLower(name);
 #else
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   if (LocaleNCompare("IM_MOD_",tag,7) == 0)
     (void) CopyMagickString(name,tag,MaxTextExtent);
   else
@@ -1572,7 +1570,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
 {
 #if !defined(MAGICKCORE_BUILD_MODULES)
   {
-    extern unsigned long
+    extern size_t
       analyzeImage(Image **,const int,const char **,ExceptionInfo *);
 
     ImageFilterHandler
@@ -1583,7 +1581,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
       image_filter=analyzeImage;
     if (image_filter != (ImageFilterHandler *) NULL)
       {
-        unsigned long
+        size_t
           signature;
 
         signature=image_filter(image,argc,argv,exception);
